using System;
using System.Collections.Generic;
using System.Text;
using System.Data.OleDb;
using System.Data;
using System.Data.Common;
using MySql.Data.MySqlClient;

namespace Utils.DataBase
{
    /// <summary>
    /// Abstract root for a SQL queries Manager.
    /// </summary>
    public abstract class DbSql
    {
        #region Variables
        /// <summary>
        /// Connection string of the Access database
        /// </summary>
        protected string vConnectionString = "";
        /// <summary>
        /// Compete ConnectionString
        /// </summary>
        protected string vFullString = "";
        /// <summary>
        /// SQL Query
        /// </summary>
        protected string vSqlQuery = "";
        #endregion

        #region Methods
        /// <summary>
        /// Returns the value of the last ID inserted in the Database
        /// </summary>
        /// <returns>Object which contains the las primary key</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        protected abstract object LastID();
        /// <summary>
        /// Execute a Selection query
        /// </summary>
        /// <param name="_SqlQuery">The Selection SQL query</param>
        /// <param name="_ds">DataSet wich is filled by the query</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <returns>A DataSet which contains the data of the SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract int ExecuteSql(string _SqlQuery, ref DataSet _ds, bool _IsManagedConnection);
        /// <summary>
        /// Execute a Selection query
        /// </summary>
        /// <param name="_SqlQuery">The Selection SQL query</param>
        /// <param name="_ds">DataSet wich is filled by the query</param>
        /// <returns>A DataSet which contains the data of the SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract int ExecuteSql(string _SqlQuery, ref DataSet _ds);
        /// <summary>
        /// Execute a Selection query
        /// </summary>
        /// <param name="_ds">DataSet wich is filled by the query</param>
        /// <returns>A DataSet which contains the data of the SELECT</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract int ExecuteSql(ref DataSet _ds);

        /// <summary>
        /// Execute a query which does return only one value (SELECT MAX, SELECT COUNT, ...)
        /// </summary>
        /// <param name="_SqlQuery">The SQL query</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <returns>The sole value of the query</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract object ExecuteScalar(string _SqlQuery, bool _IsManagedConnection);
        /// <summary>
        /// Execute a query which does return only one value (SELECT MAX, SELECT COUNT, ...)
        /// </summary>
        /// <param name="_SqlQuery">La requt SQL</param>
        /// <returns>The sole value of the query</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract object ExecuteScalar(string _SqlQuery);
        /// <summary>
        /// Execute a query which does return only one value (SELECT MAX, SELECT COUNT, ...)
        /// </summary>
        /// <returns>The sole value of the query</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract object ExecuteScalar();

        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>
        /// <param name="_SqlQuery">Sql query</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <returns>Return the number of rows that the command modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract int ExecuteNonQuery(string _SqlQuery, bool _IsManagedConnection);
        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>
        /// <param name="_SqlQuery">Sql query</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <param name="_LastID">The ID modified with the procedure</param>
        /// <returns>Return the number of rows that the command modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public virtual int ExecuteNonQuery(string _SqlQuery, bool _IsManagedConnection, ref object _LastID)
        {
            int ret = ExecuteNonQuery(_SqlQuery, _IsManagedConnection);
            _LastID = LastID();
            return ret;
        }
        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>
        /// <param name="_SqlQuery">The SQL query</param>
        /// <returns>The number of rows the Query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract int ExecuteNonQuery(string _SqlQuery);
        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>
        /// <param name="_SqlQuery">The SQL query</param>
        /// <param name="_LastID">The ID modified with the procedure</param>
        /// <returns>The number of rows the Query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>        
        public virtual int ExecuteNonQuery(string _SqlQuery, ref object _LastID)
        {
            int ret = ExecuteNonQuery(_SqlQuery);
            _LastID = LastID();
            return ret;
        }
        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>        
        /// <returns>Return the number of rows that the command modified</returns>        
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public abstract int ExecuteNonQuery();
        /// <summary>
        /// Execute a SQL command wich does not receive arguments (INSERT, DELETE, UPDATE)
        /// </summary>
        /// <param name="_LastID">The ID modified with the procedure</param>  
        /// <returns>The number of rows the Query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public virtual int ExecuteNonQuery(ref object _LastID)
        {
            int ret = ExecuteNonQuery();
            _LastID = LastID();
            return ret;
        }

        /// <summary>
        /// Prepare the use of a stored procedure
        /// </summary>
        /// <param name="_ProcedureName">Procedure name</param>
        /// <param name="_IsManagedConnection">If TRUE, the connection is automatically opened and closed. If FALSE, use manually Open() and Close()</param>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="ArgumentException"> The value was not a valid System.Data.OleDb.OleDbCommand.CommandType.</exception>
        public abstract void PrepareProcedure(string _ProcedureName, bool _IsManagedConnection);
        /// <summary>
        /// Prepare the use of a stored procedure
        /// </summary>
        /// <param name="_ProcedureName">Procedure name</param>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="ArgumentException"> The value was not a valid System.Data.OleDb.OleDbCommand.CommandType.</exception>
        public abstract void PrepareProcedure(string _ProcedureName);

        /// <summary>
        /// Execute the Selection Procedure prepared with PrepareProcedure
        /// </summary>
        /// <param name="_ds">DataSet qui sera rempli par le rsultat de la requte</param>
        /// <returns>The number of rows that the query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public abstract int ExecuteSqlProcedure(ref DataSet _ds);
        /// <summary>
        /// Execute the Scalar Stored Procedure prepared with PrepareProcedure
        /// </summary>
        /// <returns>The sole value of the query</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public abstract object ExecuteScalarProcedure();
        /// <summary>
        /// Execute a Stored Procedure which does not return results (INSERT, UPDATE).
        /// </summary>
        /// <remarks>Please prepare this method by calling PrepareProcedure before</remarks>
        /// <returns>The number of rows that the query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public abstract int ExecuteNonQueryProcedure();
        /// <summary>
        /// Execute a Stored Procedure which does not return results (INSERT, UPDATE).
        /// </summary>
        /// <remarks>Please prepare this method by calling PrepareProcedure before</remarks>
        /// <param name="_LastID">Last modified ID</param>
        /// <returns>The number of rows that the query modified</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ProcedureNonPreparedException">Si on tente d'excut la procdure alors qu'elle n'est pas prpare</exception>
        public abstract int ExecuteNonQueryProcedure(ref object _LastID);

        /// <summary>
        /// Open a connection to the database
        /// </summary>
        public abstract bool Open();
        /// <summary>
        /// Close the running connection
        /// </summary>
        public abstract void Close();
        #endregion

        #region Properties
        /// <summary>
        /// Gets the connection string of the Access database
        /// </summary>
        public string ConnectionString { get { return vConnectionString; } }
        /// <summary>
        /// Gets the state of the Database connection
        /// </summary>
        public abstract ConnectionState ConnectionState { get; }
        /// <summary>
        /// Gets the exception which was thrown or NULL if none exception was thrown. Once the exception was readed, it's replaced by NULL
        /// </summary>
        public abstract Exception OpenError { get; }
        /// <summary>
        /// Gets or sets the SQL query of this instance
        /// </summary>
        public abstract string SqlQuery { get;set; }
        /// <summary>
        /// Gets or Sets the parameters of the stored procedure
        /// </summary>
        public virtual DbParameterCollection Parameters { get { return null; } }
        #endregion
    }
    /// <summary>
    /// Exception thrown when an execution error occured when a stored procedure is not prepared
    /// </summary>
    public class ProcedureNonPreparedException : Exception
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public ProcedureNonPreparedException() : base("The stored procedure cannot be processed!") { }
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="Message">Message d'erreur de l'exception</param>
        public ProcedureNonPreparedException(string Message) : base(Message) { }
    }
    /// <summary>
    /// This exception is thrown if the asked Sql query does not have the data information (Priori chech of contraints)
    /// </summary>
    public class SqlDataException : Exception
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public SqlDataException() : base("Data is missing for the execution of the Sql Query") { }
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_Message">Error message</param>
        public SqlDataException(string _Message) : base(_Message) { }
    }
}
